<?php
/*
    Plugin Name: Formbot Form Builder
    Plugin URI: https://www.formbot.com
    Description: Plugin to link your FormBot account to your Wordpress account. Directly embed your FormBot forms into your posts and pages with blocks or from the editor. List all your forms, create new one or edit the existing one and view all form submissions right from your wordpress admin panel.
    Version: 1.0.0
    Author: formbot.com
    Author URI: https://www.formbot.com
    License: GNU General Public License v3
*/

add_action('plugins_loaded', function(){
    if ( !defined('FORMBOT_URL') ) {
        define ( 'FORMBOT_URL', sanitize_url('https://app.formbot.com') );
    }
    if ( !defined('FORMBOT_ASSET_VERSION') ) {
        define ( 'FORMBOT_ASSET_VERSION', 9 );
    }

    $page_html = '<div id="formbot_container"></div>';
    $page_html .= '<input type="hidden" id="formbot_access_token" value="' . get_option('formbot_access_token') . '">';

    if ( !defined('FORMBOT_PAGE_HTML') ) {
        define ( 'FORMBOT_PAGE_HTML', $page_html );
    }
}, 0);

// plugin uninstallation
register_uninstall_hook( __FILE__, 'formbot_uninstall' );
function formbot_uninstall() {
    //delete options on uninstall
    delete_option( 'formbot_access_token' );
    delete_option( 'formbot_client_id' );
    delete_option( 'formbot_client_secret' );
}

 /*
 * Load css 
 */
add_action( 'admin_init','formbot_enqueue_scripts');
function formbot_enqueue_scripts() {
    wp_register_style( 'formbot',plugins_url('css/formbot.css?version=' . FORMBOT_ASSET_VERSION, __FILE__) );
    wp_enqueue_style( 'formbot' );

    wp_enqueue_script( 'zoid', plugins_url('js/zoid.min.js?version=' . FORMBOT_ASSET_VERSION, __FILE__));


    wp_enqueue_script( 'formbot_custom', plugins_url( 'js/custom.js', __FILE__ ));
}

 /*
 * Add formbot button to editor toolbar
 */
add_action('init', 'formbot_buttons' );

function formbot_buttons() {
    if ( ( current_user_can('edit_posts') || current_user_can('edit_pages') ) && get_user_option('rich_editing') ) {
        add_filter("mce_external_plugins", 'formbot_add_buttons');
        add_filter('mce_buttons', 'formbot_register_buttons');
    }
    register_block_type( __DIR__ . '/js' );
}

/* Load plugin js */
function formbot_add_buttons($plugin_array) {
    $plugin_array['formbot'] = plugins_url('js/formbot.js?version=' . FORMBOT_ASSET_VERSION, __FILE__);
    return $plugin_array;
}

function formbot_register_buttons($buttons) {
    array_push($buttons, "|", "formbot_form_picker");
    return $buttons;
}

/*
* Register formbot settings
*/
add_action( 'admin_menu', 'formbot_register_menu' );
function formbot_register_menu() {
    $icon = 'dashicons-format-aside';
    add_menu_page( 'Form Bot', 'Form Bot', 'manage_options', 'form-bot', 'formbot_admin_menu_page', $icon, 85);

    $is_access_token_generated = !empty(get_option('formbot_access_token')) && !empty(get_option('formbot_client_id')) && !empty(get_option('formbot_client_secret'));

    $is_access_token_valid = false;
    $access_token = get_option('formbot_access_token');

    if (!empty($access_token)) {

        //Validate access token
        $wp_request_url = FORMBOT_URL . '/api/validate-access-token';

        $wp_request_headers = array(
            'Authorization' => 'Bearer ' . $access_token
        );

        $response = wp_remote_request(
            $wp_request_url,
                array(
                    'method'    => 'GET',
                    'headers'   => $wp_request_headers
                )
        );
        $is_access_token_valid = json_decode(wp_remote_retrieve_body( $response ))->valid;
    }

    if ($is_access_token_generated && $is_access_token_valid) {
        add_submenu_page(
            'form-bot'
            , __( 'Formbot Form Builder', 'formbot' )
            , 'Form Builder'
            , 'manage_options'
            , 'formbot-form-builder'
            , 'formbot_form_builder_callback'
        );

        add_submenu_page(
            'form-bot'
            , __( 'My Forms', 'formbot' )
            , 'My Forms'
            , 'manage_options'
            , 'formbot-my-forms'
            , 'formbot_my_forms_callback'
        );

        add_submenu_page(
            'form-bot' 
            , __( 'Submissions', 'formbot' )
            , 'Submissions'
            , 'manage_options'
            , 'formbot-submissions'
            , 'formbot_submissions_callback'
        );
    }

    add_submenu_page(
        'form-bot' 
        , __( 'Settings', 'formbot' )
        , 'Settings'
        , 'manage_options'
        , 'form-bot'
        , 'formbot_admin_menu_page'
    );
}

function formbot_remove_main_menu_link( $submenu_file ) {

    global $plugin_page;

    $hidden_submenus = array(
        'form-bot' => true,
    );

    // Select another submenu item to highlight (optional).
    if ( $plugin_page && isset( $hidden_submenus[ $plugin_page ] ) ) {
        $submenu_file = 'form-bot';
    }

    // Hide the submenu.
    foreach ( $hidden_submenus as $submenu => $unused ) {
        remove_submenu_page( 'form-bot', $submenu );
    }

    return $submenu_file;
}
add_filter( 'submenu_file', 'formbot_remove_main_menu_link' );

function formbot_my_forms_callback()
{
    echo FORMBOT_PAGE_HTML;
    echo '<input type="hidden" id="formbot_url" value="' . FORMBOT_URL . '/my-forms?iframe=true">';
}

function formbot_form_builder_callback()
{
    echo '<style type="text/css">
            .update-nag, #wpfooter, #adminmenumain {
                display: none;
            }
            #wpcontent, #wpfooter {
                margin-left: 0;
                padding-left: 0;
            }
            #wpbody-content {
                padding-bottom: 0;
            }
        </style>';
    echo FORMBOT_PAGE_HTML;

    if (!empty($_GET['token'])) {
        echo '<input type="hidden" id="formbot_url" value="' . FORMBOT_URL . '/forms/edit/' . esc_attr($_GET['id']) . '?token=' . esc_attr($_GET['token']) . '">';
    } else {
        echo '<input type="hidden" id="formbot_url" value="' . FORMBOT_URL . '/my-forms?create_form=true&iframe=true">';
    }
}

function formbot_submissions_callback()
{
    echo FORMBOT_PAGE_HTML;
    echo '<input type="hidden" id="formbot_url" value="' . FORMBOT_URL . '/submissions?iframe=true">';
}

function formbot_admin_menu_page(){
?> <div class="wrap">
<?php
    if (!empty($_GET['code'])) {
        $url = FORMBOT_URL . '/oauth/token';
        $redirect_uri = admin_url( 'admin.php?page=form-bot' );
        $response = wp_remote_post( $url, [
            'method'      => 'POST',
            'body'        => [
                    'grant_type' => 'authorization_code',
                    'client_id' => get_option('formbot_client_id'),
                    'client_secret' => get_option('formbot_client_secret'),
                    'redirect_uri' => $redirect_uri,
                    'code' => sanitize_text_field($_GET['code']),
                ]
            ]
        );

        if ($response['response']['code'] == 200 && !empty($response['body'])) {
            $body = json_decode($response['body'], true);
            if(get_option('formbot_access_token')){
                update_option('formbot_access_token', sanitize_text_field($body['access_token']));
            }
            else {
                add_option('formbot_access_token', sanitize_text_field($body['access_token']));
            }

            ?>

            <div class="notice notice-success is-dismissible">
                <p><?php _e( 'Success! Access token generated successfully.', 'formbot' ); ?></p>
            </div>
        <?php
        } else {
            echo  _e( 'Authorization failed. Please check the credentials', 'formbot'  );
        }
    }

    $client_id = get_option('formbot_client_id');
    $client_secret = get_option('formbot_client_secret');

    $is_access_token_valid = false;
    $access_token = get_option('formbot_access_token');

    if (!empty($access_token)) {
        $wp_request_url = FORMBOT_URL . '/api/validate-access-token';

        $wp_request_headers = array(
            'Authorization' => 'Bearer ' . $access_token
        );

        $response = wp_remote_request(
            $wp_request_url,
                array(
                    'method'    => 'GET',
                    'headers'   => $wp_request_headers
                )
        );
        $is_access_token_valid = json_decode(wp_remote_retrieve_body( $response ))->valid;
    }
?>
    <?php if(empty($access_token)) { ?>
        <div class="notice notice-error is-dismissible">
            <p><?php _e( 'Access token not set!', 'formbot' ); ?></p>
        </div>
    <?php } ?>

    <?php if(!$is_access_token_valid) { ?>
        <div class="notice notice-error is-dismissible">
            <p><?php _e( 'Access token is expired or invalid. Please, generate new one.', 'formbot' ); ?></p>
        </div>
    <?php } ?>
        <h1><?php _e( 'Formbot Request for access token', 'formbot' ); ?></h1>
        <form method="post" action="<?php echo admin_url( 'admin.php' ); ?>">
            <input type="hidden" name="action" value="formbot_authorize" />
            <table class="form-table">
                <tr valign="top">
                    <th scope="row"><label for="formbot_client_id"><?php _e('Client ID', 'formbot' )?></label></th>
                    <td><input type="text" id="formbot_client_id" rows="3" name="formbot_client_id" class="large-text code" value="<?php echo esc_attr($client_id); ?>" required></td>
                </tr>
                <tr valign="top">
                    <th scope="row"><label for="formbot_client_secret"><?php _e('Client Secret', 'formbot' )?></label></th>
                    <td><input type="text" id="formbot_client_secret" rows="3" name="formbot_client_secret" class="large-text code" value="<?php echo esc_attr($client_secret); ?>" required></td>
                </tr>
            </table>
            <input name="submit" class="button button-primary" type="submit" value="<?php esc_attr_e( 'Request access token', 'formbot'  ); ?>" />
        </form>

        <h2><?php _e( 'How to generate access token?', 'formbot' ); ?></h2>
        <ol>
            <li><?php _e( 'Login to your', 'formbot' ); ?> <a target="_blank" href="<?php echo FORMBOT_URL . '/login'; ?>"><?php _e( 'Formbot', 'formbot' ); ?></a> <?php _e( 'account.', 'formbot' ); ?></li>
            <li><?php _e( 'Go to', 'formbot' ); ?> <a target="_blank" href="<?php echo FORMBOT_URL . '/manage/clients'; ?>"><?php _e( 'Manage Client', 'formbot' ); ?> </a> <?php _e( 'page', 'formbot' ); ?></li>
            <li><?php _e( 'Create a new client with redirect url', 'formbot' ); ?><code><?php echo admin_url( 'admin.php?page=form-bot' ); ?></code></li>
            <li><?php _e( 'Fill the above form with generarated', 'formbot' ); ?> <code><?php _e( 'Client Id', 'formbot' ); ?></code> <?php _e( 'and', 'formbot' ); ?><code><?php _e( 'Secret Key', 'formbot' ); ?></code></li>
            <li><?php _e( 'Click on', 'formbot' ); ?><code><?php _e( 'Request access token', 'formbot' ); ?></code> <?php _e( 'and you will be redirected to authorization page', 'formbot' ); ?></li>
            <li><?php _e( 'Click on', 'formbot' ); ?><code><?php _e( 'Authorize', 'formbot' ); ?></code> <?php _e( 'and you are all set.', 'formbot' ); ?></li>
        </ol>

        <h2><?php _e( 'How to emebed a Form on your posts/pages?', 'formbot' ); ?></h2>
        <h4><?php _e( 'Method 1: Use formbot tool in classic editor', 'formbot' ); ?></h4>
        <ol>
            <li><?php _e( 'Go to the edit page of your post/page you want to embed your form.', 'formbot' ); ?></li>
            <li><?php _e( 'If you are using Gutenberg editor, add a Classic block to your post/page', 'formbot' ); ?></li>
            <li><?php _e( 'Click on Formbot’s Form Picker tool.', 'formbot' ); ?></li>
            <li><?php _e( 'Select your form from your Formbot account.', 'formbot' ); ?></li>
            <li><?php _e( 'Click on continue and the link for your selected form will be added to the editor.', 'formbot' ); ?></li>
            <li><?php _e( 'The form will be automatically embedded in your link.', 'formbot' ); ?></li>
            <li><?php _e( 'Save/Preview it.', 'formbot' ); ?></li>
        </ol>
        <h4><?php _e( 'Method 2: Use formbot custom block', 'formbot' ); ?></h4>
        <ol>
            <li><?php _e( 'Go to the edit page of your post/page you want to embed your form.', 'formbot' ); ?></li>
            <li><?php _e( 'If you are using Gutenberg editor, search "formbot" in the block finder search box and select "Formbot" custom block.', 'formbot' ); ?></li>
            <li><?php _e( 'Select the form you want to embed from the dropdown and it will be automatically embedded to your post/page.', 'formbot' ); ?></li>
        </ol>
        <h4><?php _e( 'Method 3: Use Embed block', 'formbot' ); ?></h4>
        <ol>
            <li><?php _e( 'You can use the Embed block of wordpress and paste the link of your form and it will be embedded in your post/page.', 'formbot' ); ?></li>
        </ol>

    </div>
<?php
    
}

 /*
 * Function to fetch all forms from formbot api
 */
add_action( 'wp_ajax_formbot_get_forms', 'formbot_get_forms' );

function formbot_get_forms() {
    $token = get_option('formbot_access_token');
    $wp_request_url = FORMBOT_URL . '/api/get_user_forms';

    $wp_request_headers = array(
        'Authorization' => 'Bearer ' . $token
    );

    $response = wp_remote_request(
        $wp_request_url,
            array(
                'method'    => 'GET',
                'headers'   => $wp_request_headers
            )
    );
    wp_send_json(json_decode($response['body']));
}

function formbot_custom_js() {
    global $current_screen;

    if ( !empty($current_screen->base) && in_array($current_screen->base, ['post', 'site-editor'])) {
        echo "<script type='text/javascript'>
                window.formbot_forms = '';
            </script>";
    }

    echo '<script type="text/javascript">
            const formbot_ajaxurl = "' . admin_url('admin-ajax.php') . '";
            const formbot_adminurl = "' . admin_url('admin.php') . '";
        </script>';
}
add_action( 'admin_head', 'formbot_custom_js' );

add_action( 'admin_action_formbot_authorize', 'formbot_authorize' );

function formbot_authorize()
{
    $redirect_uri = admin_url( 'admin.php?page=form-bot' );

    update_option('formbot_client_id', sanitize_text_field($_POST['formbot_client_id']));
    update_option('formbot_client_secret', sanitize_text_field($_POST['formbot_client_secret']));

    $query = http_build_query([
        'client_id' => sanitize_text_field($_POST['formbot_client_id']),
        'redirect_uri' => $redirect_uri,
        'response_type' => 'code',
        'scope' => '',
        'state' => '',
    ]);
 
    return wp_redirect(FORMBOT_URL . '/oauth/authorize?'.$query);

    exit();
}